@@ -10,13 +10,20 @@ def raise_for_status(resp):
         resp["data"] = json.loads(resp["data"].decode())
 
     def handle_400():
-        if resp["data"]["error"]["message"] == "These users are not friends":
-            raise NotFriendsError("These users are not friends")
-        elif resp["data"]["error"]["message"] == "\"Users are already friends!\"":
-            raise AlreadyFriendsError("Users are already friends!")
+        if "error" in resp["data"]:
+            if resp["data"]["error"]["message"] == "These users are not friends":
+                raise NotFriendsError("These users are not friends")
+            elif resp["data"]["error"]["message"] == "\"Users are already friends!\"":
+                raise AlreadyFriendsError("Users are already friends!")
+        elif "verified" in resp["data"]:
+            raise InvalidTwoFactorAuth("2FactorAuth code is invalid.")
 
     def handle_401():
-        raise IncorrectLoginError(resp["data"]["error"]["message"])
+        if "requiresTwoFactorAuth" in resp["data"]["error"]["message"]\
+            or "Requires Two-Factor Authentication" in resp["data"]["error"]["message"]:
+            raise RequiresTwoFactorAuthError("Account is 2FactorAuth protected.")
+        elif "Invalid Username or Password" in resp["data"]["error"]["message"]:
+            raise IncorrectLoginError(resp["data"]["error"]["message"])
 
     def handle_404():
         msg = ""
@@ -37,7 +44,7 @@ def raise_for_status(resp):
 
     if resp["status"] in switch: switch[resp["status"]]()
     if resp["status"] != 200: raise GeneralError("Unhandled error occured: "+str(resp["data"]))
-    if "requiresTwoFactorAuth" in resp["data"]: raise TwoFactorAuthNotSupportedError("2FA is not supported yet.")
+    if "requiresTwoFactorAuth" in resp["data"]: raise RequiresTwoFactorAuthError("Account is 2FactorAuth protected.")
 
 class ACall:
     def __init__(self, loop=asyncio.get_event_loop(), verify=True):
@@ -62,9 +69,9 @@ class ACall:
         await self.session.close()
         self.session = None
 
-    async def call(self, path, method="GET", headers={}, params={}, json={}, no_auth=False):
+    async def call(self, path, method="GET", headers={}, params={}, json={}, no_auth=False, verify=True):
         if no_auth:
-            return await self._call(path, method, headers, params, json)
+            return await self._call(path, method, headers, params, json, verify)
 
         if self.apiKey == None:
             async with self.session.get("https://api.vrchat.cloud/api/1/config", verify_ssl=self.verify) as resp:
@@ -89,20 +96,20 @@ class ACall:
                 try: json = await resp.json()
                 except: json = None
 
-                resp = {"status": resp.status, "data": json if not json == None else content}
-                raise_for_status(resp)
+                resp = {"status": resp.status, "response": resp, "data": json if not json == None else content}
+                if verify: raise_for_status(resp)
                 return resp
 
             json = await resp.json()
             status = resp.status
 
-        resp = {"status": status, "data": json}
+        resp = {"status": status, "response": resp, "data": json}
 
-        print(method + " request at " + path + " -- " + status)
-        raise_for_status(resp)
+        print(method + " request at " + path + " -- " + str(status))
+        if verify: raise_for_status(resp)
         return resp
 
-    async def _call(self, path, method="GET", headers={}, params={}, json={}):
+    async def _call(self, path, method="GET", headers={}, params={}, json={}, verify=True):
         h = {
             "user-agent": "",
         }
@@ -133,15 +140,15 @@ class ACall:
                     try: json = await resp.json()
                     except: json = None
 
-                    return {"status": resp.status, "data": json if not json == None else content}
+                    return {"status": resp.status, "response": resp, "data": json if not json == None else content}
 
                 json = await resp.json()
                 status = resp.status
 
-            resp = {"status": status, "data": json}
+            resp = {"status": status, "response": resp, "data": json}
 
-            print(method + " request at " + path + " -- " + status)
-            raise_for_status(resp)
+            print(method + " request at " + path + " -- " + str(status))
+            if verify: raise_for_status(resp)
             return resp
 
 class Call:
@@ -159,11 +166,11 @@ class Call:
         self.session = requests.Session()
         self.b64_auth = None
 
-    def call(self, path, method="GET", headers={}, params={}, json={}, no_auth=False):
+    def call(self, path, method="GET", headers={}, params={}, json={}, no_auth=False, verify=True):
         headers["user-agent"] = ""
 
         if no_auth:
-            return self._call(path, method, headers, params, json)
+            return self._call(path, method, headers, params, json, verify)
 
         if self.b64_auth == None:
             raise NotAuthenticated("Tried to do authenticated request without setting b64 auth (Call.set_auth(b64_auth))!")
@@ -192,18 +199,18 @@ class Call:
             except: json = None
 
             print(method + " request at " + path + " -- " + str(resp.status_code))
-            resp = {"status": resp.status_code, "data": json if not json == None else resp.content}
+            resp = {"status": resp.status_code, "response": resp, "data": json if not json == None else resp.content}
 
-            raise_for_status(resp)
+            if verify: raise_for_status(resp)
             return resp
 
         print(method + " request at " + path + " -- " + str(resp.status_code))
-        resp = {"status": resp.status_code, "data": resp.json()}
+        resp = {"status": resp.status_code, "response": resp, "data": resp.json()}
 
-        raise_for_status(resp)
+        if verify: raise_for_status(resp)
         return resp
 
-    def _call(self, path, method="GET", headers={}, params={}, json={}):
+    def _call(self, path, method="GET", headers={}, params={}, json={}, verify=True):
         if self.apiKey == None:
             resp = requests.get("https://api.vrchat.cloud/api/1/config", headers=headers, verify=self.verify)
             assert resp.status_code == 200
@@ -227,13 +234,13 @@ class Call:
             except: json = None
 
             print(method + " request at " + path + " -- " + str(resp.status_code))
-            resp = {"status": resp.status_code, "data": json if not json == None else resp.content}
+            resp = {"status": resp.status_code, "response": resp, "data": json if not json == None else resp.content}
 
-            raise_for_status(resp)
+            if verify: raise_for_status(resp)
             return resp
 
         print(method + " request at " + path + " -- " + str(resp.status_code))
-        resp = {"status": resp.status_code, "data": resp.json()}
+        resp = {"status": resp.status_code, "response": resp, "data": resp.json()}
 
-        raise_for_status(resp)
+        if verify: raise_for_status(resp)
         return resp
